--	Copyright (c) 2008-2009, Quantorsht
--	All rights reserved.
--	Redistribution and use in source and binary forms, with or without modification, are permitted provided
--	that the following conditions are met:
--	*	Redistributions of source code must retain the above copyright notice, this list of conditions and the
--		following disclaimer.
--	*	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
--		following disclaimer in the documentation and/or other materials provided with the distribution.
--	*	Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to endorse or promote
--		products derived from this software without specific prior written permission.
--	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
--	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
--	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
--	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
--	USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

local ChosenTitlesList -- TitleIds stored in gapless order
local RandomTitlesList -- TitleIds stored in gapless order, random order from ChosenTitlesList
local RandomTitleIndex -- Index into RandomTitlesList

TitleTronic = LibStub( "AceAddon-3.0" ):NewAddon( "TitleTronic",
												"AceConsole-3.0",
												"AceEvent-3.0",
												"AceTimer-3.0" )

-- Since handler references TitleTronic, it needs to exist after we instantiate TitleTronic.
local Options =
{
	type = "group",
	name = "TitleTronic",
	handler = TitleTronic,
	desc = "TitleTronic options.",
	args =
	{
		frequency =
		{
			type = "range",
			name = "Update frequency",
			desc = "How often to update character's title in seconds.",
			order = 0,
			width = "full",
			min = 0,				-- 0 seconds: off
			max = 1800,				-- 30 minutes
			step = 1,
			bigStep = 30,
			get = "GetUpdateFrequency",
			set = "SetUpdateFrequency"
		},

		list =
		{
			type = "group",
			name = "Titles",
			dialogInline = true,
			args = {}
		},
	}
}

local Defaults =
{
	char =
	{
		frequency = 180,
		selectedTitles = {}
	}
}

function TitleTronic:GetKnownTitles()
	local		TitleId
	local		TitleName
	local		TitleKnown
	local		KnownTitles

	KnownTitles = {}

	-- Record all known titles.
	for TitleId = 1, GetNumTitles() do
		TitleName = GetTitleName( TitleId )

		TitleKnown = ( IsTitleKnown( TitleId ) == 1 )
		if ( TitleName ) then
			Options.args.list.args[ tostring( TitleId ) ] = { type = "toggle", name = TitleName, get = "GetTitleState", set = "SelectTitle", disabled = "IsTitleDisabled" }
		end

		KnownTitles[ TitleId ] = TitleKnown and TitleName
	end

	-- Validate self.db.char.selectedTitles against KnownTitles.
	-- There seems to be a problem where the title information isn't always available from the server
	-- when we're logging in.  So for now, don't validate.

--	for TitleId = 1, #KnownTitles do
--		self.db.char.selectedTitles[ TitleId ] = KnownTitles[ TitleId ] and ( self.db.char.selectedTitles[ TitleId ] or false )
--	end
end

function TitleTronic:UpdateKnownTitles()
	-- Notify the registry that things might have changed.

	LibStub( "AceConfigRegistry-3.0" ):NotifyChange( "TitleTronic" )
end

function TitleTronic:GenerateRandomTitleList()
	local		K
	local		V
	local		I

	RandomTitlesList = {}
	RandomTitleIndex = 1

	for K, V in pairs( ChosenTitlesList ) do 
		RandomTitlesList[ K ] = V 
	end

	for I = 1, #RandomTitlesList do
		local	J = random( I, #RandomTitlesList )

		RandomTitlesList[ I ], RandomTitlesList[ J ] = RandomTitlesList[ J ], RandomTitlesList[ I ]
	end
end

function TitleTronic:SelectNewTitle()
	local		TitleId

	-- Leave if no titles.
	if ( #RandomTitlesList == 0 ) then
		return
	end

	SetCurrentTitle( RandomTitlesList[ RandomTitleIndex ] )

	RandomTitleIndex = RandomTitleIndex + 1
	if ( RandomTitleIndex > #RandomTitlesList ) then
		RandomTitleIndex = 1
	end
end

function TitleTronic:GenerateChosenTitlesList()
	local		TitleId

	ChosenTitlesList = {}

	-- Regenerate list of chosen titles.
	for Id, Known in pairs( self.db.char.selectedTitles ) do 
		if ( Known ) then
			tinsert( ChosenTitlesList, Id )
		end
	end

	self:GenerateRandomTitleList()
end

function TitleTronic:RescheduleTimer()
	self:CancelTimer( self.timerHandle, true )

	-- If the user has selected 0 as a frequency, don't reinstall the timer.
	if ( self.db.char.frequency > 0 ) then
		self.timerHandle = self:ScheduleRepeatingTimer( self.TitleUpdateTimer, self.db.char.frequency, self )
	end
end

function TitleTronic:ChatCommand( input )
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory( self.optionsFrame )
	else
		LibStub( "AceConfigCmd-3.0" ).HandleCommand( TitleTronic, "titletronic", "TitleTronic", input )
	end
end

function TitleTronic:OnInitialize()
	-- Called when the addon is loaded
	self.db = LibStub( "AceDB-3.0" ):New( "TitleTronicDB", Defaults, nil )

    LibStub( "AceConfig-3.0" ):RegisterOptionsTable( "TitleTronic", Options )

    self.optionsFrame = LibStub( "AceConfigDialog-3.0" ):AddToBlizOptions( "TitleTronic", "TitleTronic" )
    self:RegisterChatCommand( "titletronic", "ChatCommand" )

	self.timerHandle = nil
	ChosenTitlesList = {}
	RandomTitlesList = {}
end

function TitleTronic:OnEnable()
	-- Called when the addon is enabled

	-- It appears there is an event for when the number of titles changes
	self:RegisterEvent( "KNOWN_TITLES_UPDATE" )

	self:RegisterEvent( "PLAYER_LOGIN" )
end

function TitleTronic:OnDisable()
	-- Called when the addon is disabled
end

function TitleTronic:KNOWN_TITLES_UPDATE()
	self:UpdateKnownTitles()
end

function TitleTronic:PLAYER_LOGIN()
	self:GetKnownTitles()
	self:GenerateChosenTitlesList()
	self.timerHandle = self:ScheduleRepeatingTimer( self.TitleUpdateTimer, self.db.char.frequency, self )
end

function TitleTronic:TitleUpdateTimer()
	-- If the user has set the title to "None", then respect that.
	if GetCurrentTitle() ~= -1 then
		self:SelectNewTitle()
	end
end

function TitleTronic:GetUpdateFrequency()
	return self.db.char.frequency
end

function TitleTronic:SetUpdateFrequency( info, frequency )
	if ( self.db.char.frequency ~= frequency ) then
		self.db.char.frequency = frequency

		self:RescheduleTimer()
	end
end

function TitleTronic:GetTitleState( info )
	local TitleId = tonumber( info[ #info ] )

	return( self.db.char.selectedTitles[ TitleId ] )
end

function TitleTronic:SelectTitle( info, state )
	local TitleId = tonumber( info[ #info ] )

	self.db.char.selectedTitles[ TitleId ] = state

	self:GenerateChosenTitlesList()

	self:RescheduleTimer()
end

function TitleTronic:IsTitleDisabled( info )
	local TitleId = tonumber( info[ #info ] )

	return( IsTitleKnown( TitleId ) == 0 )
end
